"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.argumentMapper = void 0;
const logger_1 = __importDefault(require("../../logger"));
const arguments_1 = require("./arguments");
const argumentMapper = (options) => Object.keys(options)
    .filter((k) => {
    const ignoreCondition = arguments_1.ignoreOptionCombinations[k];
    if (ignoreCondition !== undefined) {
        logger_1.default.trace(`The argument mapper has an ignored combination for '${k}'`);
        if (ignoreCondition.ifNotSet &&
            options[ignoreCondition.ifNotSet] === undefined) {
            logger_1.default.warn(`Ignoring option '${k}' because it is invalid without '${ignoreCondition.ifNotSet}' also being set. This may indicate an error in your configuration`);
            return false;
        }
        logger_1.default.trace(`But it was not ignored: '${k}'`);
    }
    return true;
})
    .map((key) => {
    const thisMapping = arguments_1.argMapping[key];
    const thisValue = options[key];
    if (!thisMapping) {
        if (!arguments_1.ignoredArguments.includes(key)) {
            logger_1.default.error(`Pact-core is ignoring unknown option '${key}'`);
        }
        return [];
    }
    if (thisValue === undefined) {
        logger_1.default.warn(`The Verifier option '${key}' was was explicitly set to undefined and will be ignored. This may indicate an error in your config. Remove the option entirely to prevent this warning`);
        return [];
    }
    if ('warningMessage' in thisMapping) {
        logger_1.default.warn(thisMapping.warningMessage);
        return [];
    }
    if ('arg' in thisMapping) {
        switch (thisMapping.mapper) {
            case 'string':
                return [thisMapping.arg, `${thisValue}`];
            case 'flag':
                return thisValue ? [thisMapping.arg] : [];
            default:
                logger_1.default.pactCrash(`Option mapper for '${key}' maps to '${thisMapping.arg}' with unknown mapper type '${thisMapping.mapper}'`);
                return [];
        }
    }
    if (typeof thisMapping === 'function') {
        return thisMapping(thisValue);
    }
    logger_1.default.pactCrash(`The option mapper completely failed to find a mapping for '${key}'.`);
    return [];
})
    .reduce((acc, current) => [...acc, ...current], []);
exports.argumentMapper = argumentMapper;
//# sourceMappingURL=index.js.map